# -*- coding: utf-8 -*-
"""
Created on Tue Dec 29 23:24:35 2015

@author: Miguel Canteras
"""

import libs.bottle as bottle
from libs.bottle import route, request, response, abort
import os
import utils
import requests
from sklearn.externals import joblib
from sklearn import svm
import json

# Load SVM at startup
clf_prob = joblib.load('./model_svm_prob/emotions_svm.pkl')



# Enable CORS calls
# Based on http://paulscott.co.za/blog/cors-in-python-bottle-framework/
def enable_cors(fn):
    def _enable_cors(*args, **kwargs):
        # set CORS headers
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
        response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

        if request.method != 'OPTIONS':
            # actual request; reply with the actual response
            return fn(*args, **kwargs)

    return _enable_cors

@route('/classification/v1/emotions/ratios', method=['POST', 'OPTIONS'])
@enable_cors
def classify():

    current_feature = json.loads(request.forms.get('feature'))

    # ==============
    # CLASSIFICATION
    # ==============

    svm_pred = clf_prob.predict_proba(current_feature)

    for i in range(len(svm_pred[0])):
        svm_pred[0][i] = round(svm_pred[0][i] * 100)

    emotions_ratio = dict([('anger',svm_pred[0][0]), ('contempt',svm_pred[0][1]), ('disgust',svm_pred[0][2]), ('fear',svm_pred[0][3]), ('happiness',svm_pred[0][4]), ('sadness',svm_pred[0][5]), ('surprise',svm_pred[0][6])])
    result = dict([('Classification', emotions_ratio)])

    # ===========
    # RETURN DATA
    # ===========
    response.content_type = 'application/json'
    return result


# Run bottle internal test server when invoked directly ie: non-uxsgi mode
if __name__ == '__main__':
    bottle.run(host='0.0.0.0', port=4042, debug=True, reloader=True)
# Run bottle in application mode. Required in order to get the application working with uWSGI!
else:
    app = application = bottle.default_app()
    