# -*- coding: utf-8 -*-
"""
Created on Tue Dec 29 23:24:35 2015

@author: Miguel Canteras
"""

import libs.bottle as bottle
from libs.bottle import route, request, response, abort
import os
import emotionsUtils
import uuid
import imghdr
import shutil
import requests
import cv2
import json


def printLandmarks(img, ld):
    for k, v in ld.iteritems():
        if isinstance(v, dict) and 'x' not in v:
            printLandmarks(img, v)
        else:
            cv2.circle(img, (int(v['x']), int(v['y'])), 2, (0, 255, 0), -1)


# Enable CORS calls
# Based on http://paulscott.co.za/blog/cors-in-python-bottle-framework/
def enable_cors(fn):
    def _enable_cors(*args, **kwargs):
        # set CORS headers
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
        response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

        if request.method != 'OPTIONS':
            # actual request; reply with the actual response
            return fn(*args, **kwargs)

    return _enable_cors

@route('/emotions/v1/images', method=['POST', 'OPTIONS'])
@enable_cors
def add_image():
    url = request.forms.get('url')
    server_id = request.forms.get('serverId')

    file_path=''

    # =========
    # GET IMAGE
    # =========
    tempfolder = emotionsUtils.get_temp_folder()
    find_ext = ''

    if url is not None:
        # get image form URL
        id = str(uuid.uuid4())
        temp_path =  tempfolder + id
        result, find_ext = emotionsUtils.get_image(url, temp_path)
        if result:
            result_data = dict([('id', id), ('extension', find_ext)])
            file_path = temp_path + '.' + find_ext
        else:
            abort(404, "Image not found")

    elif server_id is not None:
        # get image from local resources
        imgsfolder = emotionsUtils.get_local_img_folder()
        temp_path =  imgsfolder + server_id + ".jpg"
        if os.path.isfile(temp_path):
            in_file = open(temp_path, 'rb')
            find_ext = imghdr.what(temp_path)
            in_file.close()
            id = str(uuid.uuid4())
            file_path = tempfolder + id + '.' + find_ext
            shutil.copy(temp_path, file_path)

            result_data = dict([('id', id), ('extension', find_ext)])
        else:
            abort(404, "Image not found")

    else:
        # image uploaded
        upload = request.files.get('files')

        # save image to temp folder
        id = str(uuid.uuid4())
        temp_path =  tempfolder + id
        upload.save(temp_path)

        # get file extension
        in_file = open(temp_path, 'rb')
        find_ext = imghdr.what(temp_path)
        in_file.close()
        if find_ext not in ('png','jpg','jpeg'):
            os.remove(temp_path)
            abort(404, "Invalid extension")
        else:
            file_path = temp_path + '.' + find_ext
            shutil.move(temp_path, file_path)

            result_data = dict([('id', id), ('extension', find_ext)])


    # ============
    # DETECT FACES
    # ============

    # Face tracking library algorithm already does this


    # ============
    # FACETRACKING
    # ============
    url = emotionsUtils.get_config_value('Landmarks', 'Server')
    fd = open(file_path, 'rb')
    files = {'files': fd}
    r = requests.post(url, files=files)
    landmarks = r.json()
    fd.close()


    # =============
    # NORMALIZATION
    # =============

    # Calculate left eye centroid
    leye_dict = landmarks['landmarks']['left_eye']
    leye_centroid_X = 0;
    leye_centroid_Y = 0;
    for k,v in leye_dict.iteritems():
        leye_centroid_X += v['x']
        leye_centroid_Y += v['y']
    leye_centroid_X /= len(leye_dict)
    leye_centroid_Y /= len(leye_dict)

    # Calculate right eye centroid
    reye_dict = landmarks['landmarks']['right_eye']
    reye_centroid_X = 0;
    reye_centroid_Y = 0;
    for k,v in reye_dict.iteritems():
        reye_centroid_X += v['x']
        reye_centroid_Y += v['y']
    reye_centroid_X /= len(reye_dict)
    reye_centroid_Y /= len(reye_dict)

    url = emotionsUtils.get_config_value('Normalization', 'Server')
    fd = open(file_path, 'rb')
    lbp_eye_distance = 50
    lbp_width = 100
    lbp_height = 120
    lbp_eye_height = 0.25
    payload = {'eye_distance':lbp_eye_distance, 'width':lbp_width, 'height':lbp_height, 'eye_height':lbp_eye_height, 'lefteye_x': leye_centroid_X, 'lefteye_y': leye_centroid_Y, 'righteye_x': reye_centroid_X, 'righteye_y': reye_centroid_Y}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    fd.close()

    id_face = str(uuid.uuid4())
    file_path_face = tempfolder + id_face + '.' + find_ext

    fd = open(file_path_face, 'wb')
    if r.status_code == 200:
        shutil.copyfileobj(r.raw, fd)
    fd.close()

    faces_data = dict([
            ('1',dict([('id',id_face),('extension',find_ext)]))
        ])


    result_data['faces'] = faces_data


    # =====================
    # LOCAL BINARY PATTERNS
    # =====================

    cvimg = cv2.imread(file_path_face)

    url = emotionsUtils.get_config_value('LBP', 'Server')

    # left eye image
    id = id_face + 'le'
    temp_path =  tempfolder + id + '.png'

    x = ((lbp_width - lbp_eye_distance) / 2) - 20
    y = lbp_height * lbp_eye_height - 25
    width = 20 + lbp_eye_distance / 2 + 5
    height = 40
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # send left eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': 8, 'radius': 1, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature = [(features[k]) for k in sorted(features, key=int)]
    fd.close()

    # right eye image
    id = id_face + 're'
    temp_path =  tempfolder + id + '.png'

    x = lbp_width - ((lbp_width - lbp_eye_distance) / 2) - lbp_eye_distance / 2 - 5
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # right left eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': 8, 'radius': 1, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature += [(features[k]) for k in sorted(features, key=int)]
    fd.close()

    # nose image
    id = id_face + 'no'
    temp_path =  tempfolder + id + '.png'

    x = (lbp_width / 2) - 25
    y = lbp_height * lbp_eye_height
    width = 50
    height = 50
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # nose eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': 8, 'radius': 1, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature += [(features[k]) for k in sorted(features, key=int)]
    fd.close()

    # mouth image
    id = id_face + 'mo'
    temp_path =  tempfolder + id + '.png'

    x = 10
    y = lbp_height * lbp_eye_height + 40
    width = lbp_width - 20
    height = 50
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # mouth eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': 8, 'radius': 1, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature += [(features[k]) for k in sorted(features, key=int)]
    fd.close()


    # =======================
    # EMOTIONS CLASSIFICATION
    # =======================

    url = emotionsUtils.get_config_value('Classification', 'Server') + '/emotions/ratios'
    payload = {'feature': json.dumps(current_feature)}
    r = requests.post(url, data=payload, stream=True)
    result_data['emotions_ratio'] = r.json()['Classification']


    # ===========================
    # ACTION UNITS CLASSIFICATION
    # ===========================

    url = emotionsUtils.get_config_value('Classification', 'Server') + '/aus/activation'
    payload = {'feature': json.dumps(current_feature)}
    r = requests.post(url, data=payload, stream=True)
    result_data['aus_activation'] = r.json()['Classification']

    print(result_data['aus_activation'])
    # ==============
    # DRAW LANDMARKS
    # ==============

    cvimg = cv2.imread(file_path)
    printLandmarks(cvimg, landmarks['landmarks'])
    cv2.imwrite(file_path, cvimg)


    # ===========
    # RETURN DATA
    # ===========
    response.content_type = 'application/json'
    return result_data



# Run bottle internal test server when invoked directly ie: non-uxsgi mode
if __name__ == '__main__':
    bottle.run(host='0.0.0.0', port=4043, debug=True, reloader=True)
# Run bottle in application mode. Required in order to get the application working with uWSGI!
else:
    app = application = bottle.default_app()
    