
# -*- coding: utf-8 -*-

import libs.bottle as bottle
from libs.bottle import route, request, response, abort
import utils
import os
import uuid
import imghdr
import shutil
from skimage.feature import hog
from skimage import data, color
import numpy

# Enable CORS calls
# Based on http://paulscott.co.za/blog/cors-in-python-bottle-framework/
def enable_cors(fn):
    def _enable_cors(*args, **kwargs):
        # set CORS headers
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
        response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

        if request.method != 'OPTIONS':
            # actual request; reply with the actual response
            return fn(*args, **kwargs)

    return _enable_cors


@route('/features/v1/HOG/images', method=['POST', 'OPTIONS'])
@enable_cors
def normalize():
    # =========
    # GET IMAGE
    # =========

    # image uploaded
    upload = request.files.get('files')

    # save image to temp folder
    folder = utils.get_temp_folder()
    id = str(uuid.uuid4())
    temp_path =  folder + id
    upload.save(temp_path)

    # get file extension
    in_file = open(temp_path, 'rb')
    find_ext = imghdr.what(temp_path)
    in_file.close()
    if find_ext not in ('png','jpg','jpeg'):
        os.remove(temp_path)
        abort(404, "Invalid extension")

    file_path = temp_path + '.' + find_ext
    shutil.move(temp_path, file_path)

    # ===============================
    # HISTOGRAM OF ORIENTED GRADIENTS
    # ===============================
    orientation = request.forms.get('orientations')
    if orientation is None:
        orientation = utils.get_config_value('Parameters', 'Orientations')

    cell_x = request.forms.get('cell_x')
    if cell_x is None:
        cell_x = utils.get_config_value('Parameters', 'Cell_x')

    cell_y = request.forms.get('cell_y')
    if cell_y is None:
        cell_y = utils.get_config_value('Parameters', 'Cell_y')

    block_x = request.forms.get('block_x')
    if block_x is None:
        block_x = utils.get_config_value('Parameters', 'Block_x')

    block_y = request.forms.get('block_y')
    if block_y is None:
        block_y = utils.get_config_value('Parameters', 'Block_y')


    image = data.load(file_path)
    image = color.rgb2gray(image)

    hog = hog(image, orientations=int(orientation), pixels_per_cell=(int(cell_x), int(cell_y)), cells_per_block=(int(block_x), int(block_y))) * 256
    values, bins = numpy.histogram(hog, bins=numpy.arange(257))

    # normalize
    values = numpy.array(values).astype(float)
    v_scaled = preprocessing.scale(values)
    
    result = dict([('HOG', dict(enumerate(hog_feature)))])

    response.content_type = 'application/json'
    return result


# Run bottle internal test server when invoked directly ie: non-uxsgi mode
if __name__ == '__main__':
    bottle.run(host='0.0.0.0', port=4043, debug=True, reloader=True)
# Run bottle in application mode. Required in order to get the application working with uWSGI!
else:
    app = application = bottle.default_app()
    
