# -*- coding: utf-8 -*-

import libs.bottle as bottle
from libs.bottle import route, request, response, abort
import utils
import os
import uuid
import imghdr
import shutil
from skimage.feature import local_binary_pattern
from skimage import data, color
import numpy

# Enable CORS calls
# Based on http://paulscott.co.za/blog/cors-in-python-bottle-framework/
def enable_cors(fn):
    def _enable_cors(*args, **kwargs):
        # set CORS headers
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
        response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

        if request.method != 'OPTIONS':
            # actual request; reply with the actual response
            return fn(*args, **kwargs)

    return _enable_cors


@route('/features/v1/LBP/images', method=['POST', 'OPTIONS'])
@enable_cors
def normalize():
    # =========
    # GET IMAGE
    # =========

    # image uploaded
    upload = request.files.get('files')

    # save image to temp folder
    folder = utils.get_temp_folder()
    id = str(uuid.uuid4())
    temp_path =  folder + id
    upload.save(temp_path)

    # get file extension
    in_file = open(temp_path, 'rb')
    find_ext = imghdr.what(temp_path)
    in_file.close()
    if find_ext not in ('png','jpg','jpeg'):
        os.remove(temp_path)
        abort(404, "Invalid extension")

    file_path = temp_path + '.' + find_ext
    shutil.move(temp_path, file_path)

    # =====================
    # LOCAL BINARY PATTERNS
    # =====================
    points = request.forms.get('n_points')
    if points is None:
        points = utils.get_config_value('Parameters', 'Points')

    radius = request.forms.get('radius')
    if radius is None:
        radius = utils.get_config_value('Parameters', 'Radius')

    method = request.forms.get('method')
    if method is None:
        method = utils.get_config_value('Parameters', 'Method')

    image = data.load(file_path)
    image = color.rgb2gray(image)

    if method == 'original':
        lbp = local_binary_pattern(image, int(points), float(radius), 'default')
        values, bins = numpy.histogram(lbp, bins=numpy.arange(257))
    elif method == 'uniform':
        lbp = local_binary_pattern(image, int(points), float(radius), 'nri_uniform')
        values, bins = numpy.histogram(lbp, bins=numpy.arange(60), density=True)
    else:
        abort(400, "Invalid method parameter")


    result = dict([('LBP', dict(enumerate(values)))])

    response.content_type = 'application/json'
    return result


# Run bottle internal test server when invoked directly ie: non-uxsgi mode
if __name__ == '__main__':
    bottle.run(host='0.0.0.0', port=4043, debug=True, reloader=True)
# Run bottle in application mode. Required in order to get the application working with uWSGI!
else:
    app = application = bottle.default_app()
    
