# -*- coding: utf-8 -*-

import libs.bottle as bottle
from libs.bottle import route, request, run, response, abort
import utils
import os
import uuid
import imghdr
import shutil
import numpy
import face_tracker


# Enable CORS calls
# Based on http://paulscott.co.za/blog/cors-in-python-bottle-framework/
def enable_cors(fn):
    def _enable_cors(*args, **kwargs):
        # set CORS headers
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
        response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

        if request.method != 'OPTIONS':
            # actual request; reply with the actual response
            return fn(*args, **kwargs)

    return _enable_cors


@route('/landmarks/v1/saragih/landmarks', method=['POST', 'OPTIONS'])
@enable_cors
def get_landmarks():
    # =========
    # GET IMAGE
    # =========

    # image uploaded
    upload = request.files.get('files')

    # save image to temp folder
    folder = utils.get_temp_folder()
    id = str(uuid.uuid4())
    file_path =  folder + id
    upload.save(file_path)

    # get file extension
    in_file = open(file_path, 'rb')
    find_ext = imghdr.what(file_path)
    in_file.close()
    if find_ext not in ('png','jpg','jpeg'):
        os.remove(file_path)
        abort(404, "Invalid extension")

    path = file_path + '.' + find_ext
    shutil.move(file_path, path)

    # ============
    # FACETRACKING
    # ============
    points = numpy.zeros(shape=(66,2))
    scale = 1.0
    face_tracker.getPoints(path, points, scale)

    landmarks = dict([
        ('left_jaw', dict([
            ('1',dict([('x',points[0][0]),('y',points[0][1])])),
            ('2',dict([('x',points[1][0]),('y',points[1][1])])),
            ('3',dict([('x',points[2][0]),('y',points[2][1])])),
            ('4',dict([('x',points[3][0]),('y',points[3][1])])),
            ('5',dict([('x',points[4][0]),('y',points[4][1])])),
            ('6',dict([('x',points[5][0]),('y',points[5][1])])),
            ('7',dict([('x',points[6][0]),('y',points[6][1])])),
            ('8',dict([('x',points[7][0]),('y',points[7][1])]))
        ])),
        ('chin', dict([
            ('1',dict([('x',points[8][0]),('y',points[8][1])]))
        ])),
        ('right_jaw', dict([
            ('1',dict([('x',points[9][0]),('y',points[9][1])])),
            ('2',dict([('x',points[10][0]),('y',points[10][1])])),
            ('3',dict([('x',points[11][0]),('y',points[11][1])])),
            ('4',dict([('x',points[12][0]),('y',points[12][1])])),
            ('5',dict([('x',points[13][0]),('y',points[13][1])])),
            ('6',dict([('x',points[14][0]),('y',points[14][1])])),
            ('7',dict([('x',points[15][0]),('y',points[15][1])])),
            ('8',dict([('x',points[16][0]),('y',points[16][1])]))
        ])),
        ('left_eyebrow', dict([
            ('1',dict([('x',points[17][0]),('y',points[17][1])])),
            ('2',dict([('x',points[18][0]),('y',points[18][1])])),
            ('3',dict([('x',points[19][0]),('y',points[19][1])])),
            ('4',dict([('x',points[20][0]),('y',points[20][1])])),
            ('5',dict([('x',points[21][0]),('y',points[21][1])])),
        ])),
        ('right_eyebrow', dict([
            ('1',dict([('x',points[22][0]),('y',points[22][1])])),
            ('2',dict([('x',points[23][0]),('y',points[23][1])])),
            ('3',dict([('x',points[24][0]),('y',points[24][1])])),
            ('4',dict([('x',points[25][0]),('y',points[25][1])])),
            ('5',dict([('x',points[26][0]),('y',points[26][1])]))
        ])),
        ('nose_bridge', dict([
            ('1',dict([('x',points[27][0]),('y',points[27][1])])),
            ('2',dict([('x',points[28][0]),('y',points[28][1])])),
            ('3',dict([('x',points[29][0]),('y',points[29][1])])),
            ('4',dict([('x',points[30][0]),('y',points[30][1])]))
        ])),
        ('nose_base', dict([
            ('1',dict([('x',points[31][0]),('y',points[31][1])])),
            ('2',dict([('x',points[32][0]),('y',points[32][1])])),
            ('3',dict([('x',points[33][0]),('y',points[33][1])])),
            ('4',dict([('x',points[34][0]),('y',points[34][1])])),
            ('5',dict([('x',points[35][0]),('y',points[35][1])]))
        ])),
        ('left_eye', dict([
            ('1',dict([('x',points[36][0]),('y',points[36][1])])),
            ('2',dict([('x',points[37][0]),('y',points[37][1])])),
            ('3',dict([('x',points[38][0]),('y',points[38][1])])),
            ('4',dict([('x',points[39][0]),('y',points[39][1])])),
            ('5',dict([('x',points[40][0]),('y',points[40][1])])),
            ('6',dict([('x',points[41][0]),('y',points[41][1])]))
        ])),
        ('right_eye', dict([
            ('1',dict([('x',points[42][0]),('y',points[42][1])])),
            ('2',dict([('x',points[43][0]),('y',points[43][1])])),
            ('3',dict([('x',points[44][0]),('y',points[44][1])])),
            ('4',dict([('x',points[45][0]),('y',points[45][1])])),
            ('5',dict([('x',points[46][0]),('y',points[46][1])])),
            ('6',dict([('x',points[47][0]),('y',points[47][1])]))
        ])),
        ('outer_mouth', dict([
            ('1',dict([('x',points[48][0]),('y',points[48][1])])),
            ('2',dict([('x',points[49][0]),('y',points[49][1])])),
            ('3',dict([('x',points[50][0]),('y',points[50][1])])),
            ('4',dict([('x',points[51][0]),('y',points[51][1])])),
            ('5',dict([('x',points[52][0]),('y',points[52][1])])),
            ('6',dict([('x',points[53][0]),('y',points[53][1])])),
            ('7',dict([('x',points[54][0]),('y',points[54][1])])),
            ('8',dict([('x',points[55][0]),('y',points[55][1])])),
            ('9',dict([('x',points[56][0]),('y',points[56][1])])),
            ('10',dict([('x',points[57][0]),('y',points[57][1])])),
            ('11',dict([('x',points[58][0]),('y',points[58][1])])),
            ('12',dict([('x',points[59][0]),('y',points[59][1])]))
        ])),
        ('inner_mouth', dict([
            ('1',dict([('x',points[60][0]),('y',points[60][1])])),
            ('2',dict([('x',points[61][0]),('y',points[61][1])])),
            ('3',dict([('x',points[62][0]),('y',points[62][1])])),
            ('4',dict([('x',points[63][0]),('y',points[63][1])])),
            ('5',dict([('x',points[64][0]),('y',points[64][1])])),
            ('6',dict([('x',points[65][0]),('y',points[65][1])]))
        ]))
        ])

    data = dict([('landmarks', landmarks)])

    response.content_type = 'application/json'
    return data

# Run bottle internal test server when invoked directly ie: non-uxsgi mode
if __name__ == '__main__':
    bottle.run(host='0.0.0.0', port=4043, debug=True, reloader=True)
# Run bottle in application mode. Required in order to get the application working with uWSGI!
else:
    app = application = bottle.default_app()
    