# -*- coding: utf-8 -*-

import libs.bottle as bottle
from libs.bottle import route, request, response, abort
import utils
import os
import uuid
import imghdr
import shutil
import math
import cv2

# Enable CORS calls
# Based on http://paulscott.co.za/blog/cors-in-python-bottle-framework/
def enable_cors(fn):
    def _enable_cors(*args, **kwargs):
        # set CORS headers
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
        response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

        if request.method != 'OPTIONS':
            # actual request; reply with the actual response
            return fn(*args, **kwargs)

    return _enable_cors


@route('/normalization/v1/face/images', method=['POST', 'OPTIONS'])
@enable_cors
def normalize():
    # =========
    # GET IMAGE
    # =========

    # image uploaded
    upload = request.files.get('files')

    # save image to temp folder
    folder = utils.get_temp_folder()
    id = str(uuid.uuid4())
    temp_path =  folder + id
    upload.save(temp_path)

    # get file extension
    in_file = open(temp_path, 'rb')
    find_ext = imghdr.what(temp_path)
    in_file.close()
    if find_ext not in ('png','jpg','jpeg'):
        os.remove(temp_path)
        abort(404, "Invalid extension")

    file_path = temp_path + '.' + find_ext
    shutil.move(temp_path, file_path)

    # =============
    # NORMALIZATION
    # =============

    eyeDistance = int(request.forms.get('eye_distance'))
    if eyeDistance is None:
        eyeDistance = int(utils.get_config_value('Parameters', 'EyeDistance'))
    lefteye_x = float(request.forms.get('lefteye_x'))
    lefteye_y = float(request.forms.get('lefteye_y'))
    righteye_x = float(request.forms.get('righteye_x'))
    righteye_y = float(request.forms.get('righteye_y'))

    src = cv2.imread(file_path)

    # Rotation
    x = righteye_x - lefteye_x
    y = righteye_y - lefteye_y
    if x!=0 and y!=0:
        angle = math.atan2(y, x);
        rows,cols,channels = src.shape
        M = cv2.getRotationMatrix2D((cols/2,rows/2),angle * 180 / math.pi,1)
        dst = cv2.warpAffine(src,M,(cols,rows))
    else:
        dst = src

    s = math.sin(-angle)
    c = math.cos(-angle)
    cx = cols/2
    cy = rows/2

    lexr = (lefteye_x-cx)*c - (lefteye_y-cy)*s + cx
    leyr = (lefteye_x-cx)*s + (lefteye_y-cy)*c + cy

    rexr = (righteye_x-cx)*c - (righteye_y-cy)*s + cx
    reyr = (righteye_x-cx)*s + (righteye_y-cy)*c + cy

    # Scaling
    distance = math.hypot((rexr - lexr), (reyr - leyr))
    if distance != 0:
        scale = eyeDistance / distance
        dst = cv2.resize(dst, None, fx=scale, fy=scale, interpolation = cv2.INTER_CUBIC)
    else:
        scale = 1.0

    # Cropping
    width = int(request.forms.get('width'))
    if width is None:
        width = int(utils.get_config_value('Parameters', 'ImageWidth'))
    height = int(request.forms.get('height'))
    if height is None:
        height = int(utils.get_config_value('Parameters', 'ImageHeight'))
    eyeHeight = float(request.forms.get('eye_height'))
    if eyeHeight is None:
        eyeHeight = float(utils.get_config_value('Parameters', 'EyeHeight'))
    x = lexr * scale - float(width)/2.0 + float(eyeDistance)/2.0
    y = leyr * scale - float(height)*eyeHeight
    cropped = dst[int(y):int(y+height), int(x):int(x+width)]

    cv2.imwrite(file_path, cropped)

    return bottle.static_file(id + '.' + find_ext, root=folder)


# Run bottle internal test server when invoked directly ie: non-uxsgi mode
if __name__ == '__main__':
    bottle.run(host='0.0.0.0', port=4045, debug=True, reloader=True)
# Run bottle in application mode. Required in order to get the application working with uWSGI!
else:
    app = application = bottle.default_app()
    
