// Global variables
var net;
var trainer;

var n_epochs_executed = 0;
var n_inputs = 1024; // feature vector size
var n_epochs = 1000; 
var n_labels = 7;   // number of labels to classify
var epoch_timer;

var lossGraph = new cnnvis.Graph();
var xLossWindow = new cnnutil.Window(100);
var wLossWindow = new cnnutil.Window(100);
var trainAccWindow = new cnnutil.Window(100);
var valAccWindow = new cnnutil.Window(100);
	
function createMatrix(rows, columns) {
	var matrix = [];
	for(var i=0; i<rows; i++) {
		matrix[i] = [];
		for(var j=0; j<columns; j++) {
			matrix[i][j] = 0;
		}
	}

	return matrix;
}

function printMatrix(m){
    var rows = m.length;
    var columns = m[0].length;
    var str = "";
	
	str += "<table>";
	str += "<tr><td></td><td>Anger</td><td>Contempt</td><td>Disgust</td><td>Fear</td><td>Happy</td><td>Sadness</td><td>Surprise</td></tr>";
    for(var i=0;i<rows;i++){
		str += "<tr>";
		switch(i) {
			case 0: str += "<td>Anger</td>"; break;
			case 1: str += "<td>Contempt</td>"; break;
			case 2: str += "<td>Disgust</td>"; break;
			case 3: str += "<td>Fear</td>"; break;
			case 4: str += "<td>Happy</td>"; break;
			case 5: str += "<td>Sadness</td>"; break;
			case 6: str += "<td>Surprise</td>"; break;
		}
        for(var j=0;j<columns;j++){			
            str += "<td>" + m[i][j] + "</td>";
        }
		str += "</tr>";
    }
	str += "</table>";
	
	return str;
}

function printMatrixNeutral(m){
    var rows = m.length;
    var columns = m[0].length;
    var str = "";
	
	str += "<table>";
	str += "<tr><td></td><td>Neutral</td><td>Anger</td><td>Contempt</td><td>Disgust</td><td>Fear</td><td>Happy</td><td>Sadness</td><td>Surprise</td></tr>";
    for(var i=0;i<rows;i++){
		str += "<tr>";
		switch(i) {
			case 0: str += "<td>Neutral</td>"; break;
			case 1: str += "<td>Anger</td>"; break;
			case 2: str += "<td>Contempt</td>"; break;
			case 3: str += "<td>Disgust</td>"; break;
			case 4: str += "<td>Fear</td>"; break;
			case 5: str += "<td>Happy</td>"; break;
			case 6: str += "<td>Sadness</td>"; break;
			case 7: str += "<td>Surprise</td>"; break;
		}
        for(var j=0;j<columns;j++){			
            str += "<td>" + m[i][j] + "</td>";
        }
		str += "</tr>";
    }
	str += "</table>";
	
	return str;
}

function train() {
	var stats;
	for(var i=0;i<featuresTraining.length;i++) {
		var x = new convnetjs.Vol(1,1,n_inputs,0.0); // a 1x1xn_inputs volume initialized to 0's.
		x.w = featuresTraining[i]; // Vol.w is just a list, it holds your data
		stats = trainer.train(x, classTraining[i]);		  
		
		var lossx = stats.cost_loss;
		var lossw = stats.l2_decay_loss;
		xLossWindow.add(lossx);
		wLossWindow.add(lossw);
	}		
}

function test() {
	var y = new convnetjs.Vol(1,1,1024,0.0); // a 1x1x1024 volume initialized to 0's.
	y.w = featuresTest[0]; // Vol.w is just a list, it holds your data

	var probability_volume = net.forward(y);
	var d = document.getElementById('test');
	
	var str = '';
	for(var i=0;i<n_labels;i++) {
		str += 'Probability that is class ' + (i+1) + ': ' + probability_volume.w[i] + '<br>';
	}
	d.innerHTML = str;
}
	
function run_epoch() {
	/*if(n_epochs == n_epochs_executed) {
		clearInterval(epoch_timer);
		return;
	}*/
		
	n_epochs_executed++
	var d = document.getElementById('epoch');
	d.innerHTML = '<b>Current epoch:</b> ' + n_epochs_executed;
	
	train();
	
	
    var xa = xLossWindow.get_average();
    var xw = wLossWindow.get_average();
    if(xa >= 0 && xw >= 0) { // if they are -1 it means not enough data was accumulated yet for estimates
      lossGraph.add(n_epochs_executed, xa + xw);
      lossGraph.drawSelf(document.getElementById("lossgraph"));
    }

	var d = document.getElementById('cost_loss');
	d.innerHTML = '<b>Cost loss:</b> ' + xa;
	
	var d = document.getElementById('l2_decay_loss');
	d.innerHTML = '<b>L2 decay loss:</b> ' + xw;
	
	// Confusion matrix
	confusion();
}

function resume_epoch() {
	epoch_timer = setInterval(run_epoch, 200);
}

function pause_epoch() {
	clearInterval(epoch_timer);
}

var dump_json = function() {
	//document.getElementById("dumpjson").value = JSON.stringify(net.toJSON());
	
	var blob = new Blob([JSON.stringify(net.toJSON())], {type: "text/plain;charset=utf-8"});
	saveAs(blob, "net.json");
}
 
var load_from_json = function(jsonString) {
	var json = JSON.parse(jsonString);
	net = new convnetjs.Net();
	net.fromJSON(json);
	reset_all();
}

var clear_graph = function() {
  lossGraph = new cnnvis.Graph(); // reinit graph too 
}

var reset_all = function() {
  update_net_param_display();

  // reinit windows that keep track of val/train accuracies
  xLossWindow.reset();
  wLossWindow.reset();
  trainAccWindow.reset();
  valAccWindow.reset();
  lossGraph = new cnnvis.Graph(); // reinit graph too
  n_epochs_executed = 0;
}

function confusion() {
	// Confusion matrix
	var confusion = createMatrix(n_labels,n_labels);
	
	for(var s=0;s<classTest.length;s++) {
		var y = new convnetjs.Vol(1,1,1024,0.0); // a 1x1x1024 volume initialized to 0's.
		y.w = featuresTest[s]; // Vol.w is just a list, it holds your data

		var probability_volume = net.forward(y);
		
		var max=-1;
		var sampleclass = 0;
		for(var i=0;i<n_labels;i++) {
			if(max < probability_volume.w[i])
			{
				max = probability_volume.w[i];
				sampleclass = i;
			}
		}
		
		confusion[classTest[s]][sampleclass] += 1;
	}
	
	var d = document.getElementById('confusion');
	d.innerHTML = printMatrix(confusion);
}

function start() {
	var layer_defs = [];
	// input layer of size 1x1x2 (all volumes are 3D)
	layer_defs.push({type:'input', out_sx:1, out_sy:1, out_depth:n_inputs});
	// some fully connected layers
	for(var i=0; i<15; i++)
	{
		layer_defs.push({type:'fc', num_neurons:50, activation:'relu'});
		layer_defs.push({type:'fc', num_neurons:20, activation:'relu'});
	}
	
	//layer_defs.push({type:'fc', num_neurons:20, activation:'relu'});
	// a softmax classifier predicting probabilities
	layer_defs.push({type:'softmax', num_classes:n_labels});
	//layer_defs.push({type:'regression', num_neurons: n_labels});
	
	 
	/*var layer_defs = [];
	layer_defs.push({type:'input', out_sx:1, out_sy:1, out_depth:n_inputs});
	layer_defs.push({type:'conv', sx:5, filters:16, stride:1, pad:2, activation:'relu'});
	layer_defs.push({type:'pool', sx:2, stride:2});
	layer_defs.push({type:'softmax', num_classes:n_labels});
*/
	// create a net out of it
	net = new convnetjs.Net();
	net.makeLayers(layer_defs);
	
	//trainer = new convnetjs.Trainer(net, {method: 'adadelta', l2_decay: 0.001, batch_size: 1});	
	//trainer = new convnetjs.Trainer(net, {method: 'adadelta', l2_decay: 0.00001, batch_size: 10});	
	//trainer = new convnetjs.Trainer(net, {method: 'adadelta', l2_decay: 0.0001, batch_size: 10});	
	trainer = new convnetjs.Trainer(net, {method: 'adadelta', l2_decay: 0.001, batch_size: 10});	
	
	//trainer = new convnetjs.Trainer(net, {method: 'adagrad', l2_decay: 0.001, l1_decay: 0.001, batch_size: 10});
	//trainer = new convnetjs.Trainer(net, {learning_rate:0.0001, method: 'sgd', momentum: 0.9, batch_size:8, l2_decay:0.001});
	
	confusion();
	
	lossGraph.drawSelf(document.getElementById("lossgraph"));
}

