# -*- coding: utf-8 -*-
"""
Created on Tue Dec 29 23:24:35 2015

@author: Miguel Canteras
"""

import os
import utils
import uuid
import imghdr
import shutil
import requests
import cv2

def get_feature(file_path, lbp_points, lbp_radius):

    # get file extension
    in_file = open(file_path, 'rb')
    find_ext = imghdr.what(file_path)
    in_file.close()
    if find_ext not in ('png','jpg','jpeg'):
        return

    # ============
    # FACETRACKING
    # ============
    url = utils.get_config_value('Landmarks', 'Server')
    fd = open(file_path, 'rb')
    files = {'files': fd}
    r = requests.post(url, files=files)
    landmarks = r.json()
    fd.close()


    # =============
    # NORMALIZATION
    # =============

    # Calculate left eye centroid
    leye_dict = landmarks['landmarks']['left_eye']
    leye_centroid_X = 0;
    leye_centroid_Y = 0;
    for k,v in leye_dict.iteritems():
        leye_centroid_X += v['x']
        leye_centroid_Y += v['y']
    leye_centroid_X /= len(leye_dict)
    leye_centroid_Y /= len(leye_dict)

    # Calculate right eye centroid
    reye_dict = landmarks['landmarks']['right_eye']
    reye_centroid_X = 0;
    reye_centroid_Y = 0;
    for k,v in reye_dict.iteritems():
        reye_centroid_X += v['x']
        reye_centroid_Y += v['y']
    reye_centroid_X /= len(reye_dict)
    reye_centroid_Y /= len(reye_dict)

    url = utils.get_config_value('Normalization', 'Server')
    fd = open(file_path, 'rb')
    lbp_eye_distance = 50
    lbp_width = 100
    lbp_height = 120
    lbp_eye_height = 0.25
    payload = {'eye_distance':lbp_eye_distance, 'width':lbp_width, 'height':lbp_height, 'eye_height':lbp_eye_height, 'lefteye_x': leye_centroid_X, 'lefteye_y': leye_centroid_Y, 'righteye_x': reye_centroid_X, 'righteye_y': reye_centroid_Y}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    fd.close()

    id_face = str(uuid.uuid4())
    tempfolder = utils.get_config_value('Folders', 'TempFolder')
    file_path_face = tempfolder + id_face + '.' + find_ext

    fd = open(file_path_face, 'wb')
    if r.status_code == 200:
        shutil.copyfileobj(r.raw, fd)
    fd.close()


    # =====================
    # LOCAL BINARY PATTERNS
    # =====================

    cvimg = cv2.imread(file_path_face)

    url = utils.get_config_value('LBP', 'Server')

    # left eye image
    id = id_face + 'le'
    temp_path =  tempfolder + id + '.png'

    x = ((lbp_width - lbp_eye_distance) / 2) - 20
    y = lbp_height * lbp_eye_height - 25
    width = 20 + lbp_eye_distance / 2 + 5
    height = 40
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # send left eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': lbp_points, 'radius': lbp_radius, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature = [(features[k]) for k in sorted(features, key=int)]
    fd.close()
    os.remove(temp_path)

    # right eye image
    id = id_face + 're'
    temp_path =  tempfolder + id + '.png'

    x = lbp_width - ((lbp_width - lbp_eye_distance) / 2) - lbp_eye_distance / 2 - 5
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # right left eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': lbp_points, 'radius': lbp_radius, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature += [(features[k]) for k in sorted(features, key=int)]
    fd.close()
    os.remove(temp_path)

    # nose image
    id = id_face + 'no'
    temp_path =  tempfolder + id + '.png'

    x = (lbp_width / 2) - 25
    y = lbp_height * lbp_eye_height
    width = 50
    height = 50
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # nose eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': lbp_points, 'radius': lbp_radius, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature += [(features[k]) for k in sorted(features, key=int)]
    fd.close()
    os.remove(temp_path)

    # mouth image
    id = id_face + 'mo'
    temp_path =  tempfolder + id + '.png'

    x = 10
    y = lbp_height * lbp_eye_height + 40
    width = lbp_width - 20
    height = 50
    cropped = cvimg[int(y):int(y+height), int(x):int(x+width)]
    cv2.imwrite(temp_path, cropped)

    # mouth eye
    fd = open(temp_path, 'rb')
    payload = {'n_points': lbp_points, 'radius': lbp_radius, 'method': 'original'}
    files = {'files': fd}
    r = requests.post(url, data=payload, files=files, stream=True)
    features = r.json()['LBP']
    current_feature += [(features[k]) for k in sorted(features, key=int)]
    fd.close()
    os.remove(temp_path)

    os.remove(file_path_face)

    return current_feature